"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.RequestBuilder = void 0;
var types_1 = require("./../../types/types");
var meta_1 = require("./meta");
var redux_store_generator_1 = require("redux-store-generator");
var sequence = 1;
var RequestBuilder = /** @class */ (function () {
    function RequestBuilder() {
        this.output = {
            status: types_1.ApiRequestStatus.IDLE,
            retriesCount: 0,
        };
        this.id = '';
        this.output.meta = meta_1.generateMeta(sequence++);
    }
    RequestBuilder.prototype.withNodeName = function (value) {
        this.output.nodeName = value;
        return this;
    };
    RequestBuilder.prototype.withMethod = function (value) {
        this.output.apiVerb = value;
        return this;
    };
    RequestBuilder.prototype.withNodeType = function (value) {
        this.output.nodeType = value;
        return this;
    };
    RequestBuilder.prototype.withParams = function (value) {
        this.output.params = value;
        return this;
    };
    RequestBuilder.prototype.withResolve = function (value) {
        this.output.resolve = value;
        return this;
    };
    RequestBuilder.prototype.withReject = function (value) {
        this.output.reject = value;
        return this;
    };
    RequestBuilder.prototype.withId = function (value) {
        this.id = value;
        return this;
    };
    RequestBuilder.prototype.build = function () {
        if (!this.output.params) {
            delete this.output.params;
        }
        var isCollection = this.output.nodeType === redux_store_generator_1.NodeType.COLLECTION_NODE;
        switch (this.output.apiVerb) {
            case 'get':
                this.output.path = "/" + this.output.nodeName;
                this.output.method = 'GET';
                break;
            case 'patch':
                this.output.path = "/" + this.output.nodeName;
                if (isCollection) {
                    this.output.path += "/" + this.id;
                }
                this.output.method = 'PATCH';
                break;
            case 'push':
                this.output.path = "/" + this.output.nodeName;
                this.output.method = 'POST';
                break;
            case 'add':
                this.output.path = "/" + this.output.nodeName;
                this.output.method = 'POST';
                break;
            case 'delete':
                this.output.path = "/" + this.output.nodeName;
                if (isCollection) {
                    this.output.path += "/" + this.id;
                }
                this.output.method = 'DELETE';
                break;
        }
        return this.output;
    };
    return RequestBuilder;
}());
exports.RequestBuilder = RequestBuilder;
//# sourceMappingURL=RequestsBuilder.js.map