import { apiConfigs, apiStatus } from './reducers';
import { ConnectionStatus, ConnectionType } from './../types/types';
import { NodeType } from 'redux-store-generator';

describe('apiConfig', () => {
    it('should be able to SET_ALL', () => {
        const state = {};
        const action = {
            type: 'SET_ALL_API_ENDPOINTS_CONFIG',
            payload: {
                appState: {
                    nodeType: NodeType.SINGLE_NODE,
                    connectionType: ConnectionType.REST,
                },
            },
        };
        expect(apiConfigs(state, action)).toEqual({
            appState: {
                nodeType: NodeType.SINGLE_NODE,
                connectionType: ConnectionType.REST,
            },
        });
    });

    it('should be able to SET', () => {
        const state = {
            appState: {
                nodeType: NodeType.SINGLE_NODE,
                connectionType: ConnectionType.REST,
            },
            products: {
                nodeType: NodeType.COLLECTION_NODE,
                connectionType: ConnectionType.NONE,
            },
        };
        const action = {
            type: 'SET_API_ENDPOINTS_CONFIG',
            payload: {
                appState: {
                    nodeType: NodeType.SINGLE_NODE,
                    connectionType: ConnectionType.NONE,
                },
            },
        };
        expect(apiConfigs(state, action)).toEqual({
            appState: {
                nodeType: NodeType.SINGLE_NODE,
                connectionType: ConnectionType.NONE,
            },
            products: {
                nodeType: NodeType.COLLECTION_NODE,
                connectionType: ConnectionType.NONE,
            },
        });
    });

    it('should not be able to OVERWRITE nodeType', () => {
        const state = {
            appState: {
                nodeType: NodeType.SINGLE_NODE,
                connectionType: ConnectionType.REST,
            },
        };
        const action = {
            type: 'SET_API_ENDPOINTS_CONFIG',
            payload: {
                appState: {
                    connectionType: ConnectionType.NONE,
                },
            },
        };
        expect(apiConfigs(state, action)).toEqual({
            appState: {
                nodeType: NodeType.SINGLE_NODE,
                connectionType: ConnectionType.NONE,
            },
        });
    });

    it('should be able to PATCH', () => {
        const state = {
            appState: {
                nodeType: NodeType.SINGLE_NODE,
                connectionType: ConnectionType.SOCKET,
            },
            products: {
                nodeType: NodeType.COLLECTION_NODE,
                connectionType: ConnectionType.NONE,
            },
        };
        const action = {
            type: 'PATCH_API_ENDPOINTS_CONFIG',
            payload: {
                appState: {
                    connectionType: ConnectionType.REST,
                    requestsPerMinute: 3,
                },
            },
        };
        expect(apiConfigs(state, action)).toEqual({
            appState: {
                nodeType: NodeType.SINGLE_NODE,
                connectionType: ConnectionType.REST,
                requestsPerMinute: 3,
            },
            products: {
                nodeType: NodeType.COLLECTION_NODE,
                connectionType: ConnectionType.NONE,
            },
        });
    });

    it('should NOT be able to PATCH nodeType', () => {
        const state = {
            appState: {
                nodeType: NodeType.SINGLE_NODE,
                connectionType: ConnectionType.REST,
            },
        };
        const action = {
            type: 'PATCH_API_ENDPOINTS_CONFIG',
            payload: {
                appState: {
                    nodeType: NodeType.COLLECTION_NODE,
                    connectionType: ConnectionType.REST,
                },
            },
        };
        expect(apiConfigs(state, action)).toEqual({
            appState: {
                nodeType: NodeType.SINGLE_NODE,
                connectionType: ConnectionType.REST,
            },
        });
    });
});

describe('apiStatus', () => {
    it('should be able to SET_ALL', () => {
        const state = {};
        const action = {
            type: 'SET_ALL_API_STATE',
            payload: {
                appState: {
                    connectionStatus: ConnectionStatus.IDLE,
                },
            },
        };
        expect(apiStatus(state, action)).toEqual({
            appState: {
                connectionStatus: ConnectionStatus.IDLE,
            },
        });
    });

    it('should be able to SET', () => {
        const state = {
            appState: {
                connectionStatus: ConnectionStatus.IDLE,
                lastRequestTS: 0,
            },
            products: {
                connectionStatus: ConnectionStatus.IDLE,
            },
        };
        const action = {
            type: 'SET_API_STATE',
            payload: {
                appState: {
                    connectionStatus: ConnectionStatus.LOADING,
                },
            },
        };
        expect(apiStatus(state, action)).toEqual({
            appState: {
                connectionStatus: ConnectionStatus.LOADING,
            },
            products: {
                connectionStatus: ConnectionStatus.IDLE,
            },
        });
    });

    it('should be able to PATCH', () => {
        const state = {
            appState: {
                connectionStatus: ConnectionStatus.IDLE,
            },
            products: {
                connectionStatus: ConnectionStatus.IDLE,
            },
        };
        const action = {
            type: 'PATCH_API_STATE',
            payload: {
                appState: {
                    connectionStatus: ConnectionStatus.RETRYING,
                },
            },
        };
        expect(apiStatus(state, action)).toEqual({
            appState: {
                connectionStatus: ConnectionStatus.RETRYING,
            },
            products: {
                connectionStatus: ConnectionStatus.IDLE,
            },
        });
    });
});
