import { useEffect } from 'react';
import { Action } from 'redux-store-generator';
import globals from '../globals';

export interface UseStoreOptions {
    connectedStore?: boolean;
}

export type StoreChangeCallback = (state: any, action: Action) => void;
export type ActionPredicate = (action: Action) => boolean;

export function useStore(
    callback: StoreChangeCallback,
    predicate?: ActionPredicate,
    options?: UseStoreOptions
) {
    const store = globals.connectedStore;

    useEffect(() => {
        const unsubscribe = store.subscribe(() => {
            const state = store.getState();
            const action = state._lastAction;

            if (!predicate || predicate(action)) {
                callback(state, action);
            }
        });

        return () => unsubscribe();
    }, [callback, predicate]);
}
