import { ApiRequestStatus } from './../../types/types';
import { generateMeta } from './meta';
import { ApiVerb, NodeType } from 'redux-store-generator';
import { ApiRequest } from '../../types/types';

let sequence = 1;

export class RequestBuilder {
    private output = {
        status: ApiRequestStatus.IDLE,
        retriesCount: 0,
    } as ApiRequest;
    private id: string = '';

    constructor() {
        this.output.meta = generateMeta(sequence++);
    }

    withNodeName(value: string) {
        this.output.nodeName = value;
        return this;
    }

    withMethod(value: ApiVerb) {
        this.output.apiVerb = value;
        return this;
    }

    withNodeType(value: NodeType) {
        this.output.nodeType = value;
        return this;
    }

    withParams(value?: Record<string, any>) {
        this.output.params = value;
        return this;
    }

    withResolve(value: any) {
        this.output.resolve = value;
        return this;
    }

    withReject(value: any) {
        this.output.reject = value;
        return this;
    }

    withId(value: string) {
        this.id = value;
        return this;
    }

    build(): ApiRequest {
        if (!this.output.params) {
            delete this.output.params;
        }

        const isCollection = this.output.nodeType === NodeType.COLLECTION_NODE;

        switch (this.output.apiVerb) {
            case 'get':
                this.output.path = `/${this.output.nodeName}`;
                this.output.method = 'GET';
                break;
            case 'patch':
                this.output.path = `/${this.output.nodeName}`;
                if (isCollection) {
                    this.output.path += `/${this.id}`;
                }
                this.output.method = 'PATCH';
                break;
            case 'push':
                this.output.path = `/${this.output.nodeName}`;
                this.output.method = 'POST';
                break;
            case 'add':
                this.output.path = `/${this.output.nodeName}`;
                this.output.method = 'POST';
                break;
            case 'delete':
                this.output.path = `/${this.output.nodeName}`;
                if (isCollection) {
                    this.output.path += `/${this.id}`;
                }
                this.output.method = 'DELETE';
                break;
        }

        return this.output;
    }
}
